"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FailedToFormatResults = exports.formatScanResults = void 0;
const types_1 = require("./types");
const path = require("path");
const common_1 = require("../../../../lib/snyk-test/common");
const constants_1 = require("../../../../lib/iac/constants");
const errors_1 = require("../../../../lib/errors");
const cloud_config_parser_1 = require("@snyk/cloud-config-parser");
const file_parser_1 = require("./file-parser");
const analytics = require("../../../../lib/analytics");
const Debug = require("debug");
const debug = Debug('iac-results-formatter');
const SEVERITIES = [common_1.SEVERITY.LOW, common_1.SEVERITY.MEDIUM, common_1.SEVERITY.HIGH];
function formatScanResults(scanResults, options) {
    try {
        // Relevant only for multi-doc yaml files
        const scannedResultsGroupedByDocId = groupMultiDocResults(scanResults);
        return scannedResultsGroupedByDocId.map((iacScanResult) => formatScanResult(iacScanResult, options.severityThreshold));
    }
    catch (e) {
        throw new FailedToFormatResults();
    }
}
exports.formatScanResults = formatScanResults;
function getFileTypeForLineNumber(fileType) {
    switch (fileType) {
        case 'yaml':
        case 'yml':
            return cloud_config_parser_1.CloudConfigFileTypes.YAML;
        case 'json':
            return cloud_config_parser_1.CloudConfigFileTypes.JSON;
        case 'tf':
            return cloud_config_parser_1.CloudConfigFileTypes.TF;
        default:
            throw new file_parser_1.UnsupportedFileTypeError(fileType);
    }
}
const engineTypeToProjectType = {
    [types_1.EngineType.Kubernetes]: constants_1.IacProjectType.K8S,
    [types_1.EngineType.Terraform]: constants_1.IacProjectType.TERRAFORM,
};
function formatScanResult(scanResult, severityThreshold) {
    const formattedIssues = scanResult.violatedPolicies.map((policy) => {
        const cloudConfigPath = scanResult.docId !== undefined
            ? [`[DocId:${scanResult.docId}]`].concat(policy.msg.split('.'))
            : policy.msg.split('.');
        let lineNumber;
        try {
            lineNumber = cloud_config_parser_1.issuesToLineNumbers(scanResult.fileContent, getFileTypeForLineNumber(scanResult.fileType), policy.msg.split('.'));
        }
        catch (_a) {
            const err = new FailedToExtractLineNumberError();
            analytics.add('error-code', err.code);
            debug('Parser library failed. Could not assign lineNumber to issue');
            lineNumber = -1;
        }
        return Object.assign(Object.assign({}, policy), { id: policy.publicId, name: policy.title, cloudConfigPath, isIgnored: false, iacDescription: {
                issue: policy.issue,
                impact: policy.impact,
                resolve: policy.resolve,
            }, severity: policy.severity, lineNumber });
    });
    const targetFilePath = path.resolve(scanResult.filePath, '.');
    return {
        result: {
            cloudConfigResults: filterPoliciesBySeverity(formattedIssues, severityThreshold),
            projectType: constants_1.projectTypeByFileType[scanResult.fileType],
        },
        isPrivate: true,
        packageManager: engineTypeToProjectType[scanResult.engineType],
        targetFile: scanResult.filePath,
        targetFilePath,
        vulnerabilities: [],
        dependencyCount: 0,
        licensesPolicy: null,
        ignoreSettings: null,
        projectName: path.basename(path.dirname(targetFilePath)),
    };
}
function groupMultiDocResults(scanResults) {
    const groupedData = scanResults.reduce((memo, result) => {
        if (memo[result.filePath]) {
            memo[result.filePath].violatedPolicies = memo[result.filePath].violatedPolicies.concat(result.violatedPolicies);
        }
        else {
            memo[result.filePath] = result;
        }
        return memo;
    }, {});
    return Object.values(groupedData);
}
function filterPoliciesBySeverity(violatedPolicies, severityThreshold) {
    if (!severityThreshold || severityThreshold === common_1.SEVERITY.LOW) {
        return violatedPolicies;
    }
    const severitiesToInclude = SEVERITIES.slice(SEVERITIES.indexOf(severityThreshold));
    return violatedPolicies.filter((policy) => severitiesToInclude.includes(policy.severity));
}
class FailedToFormatResults extends errors_1.CustomError {
    constructor(message) {
        super(message || 'Failed to format results');
        this.code = types_1.IaCErrorCodes.FailedToFormatResults;
        this.userMessage =
            'We failed printing the results, please contact support@snyk.io';
    }
}
exports.FailedToFormatResults = FailedToFormatResults;
class FailedToExtractLineNumberError extends errors_1.CustomError {
    constructor(message) {
        super(message || 'Parser library failed. Could not assign lineNumber to issue');
        this.code = types_1.IaCErrorCodes.FailedToExtractLineNumberError;
        this.userMessage = ''; // Not a user facing error.
    }
}
//# sourceMappingURL=results-formatter.js.map